{{

}}

CON
' .-----------------------------------------------------------------------------------------------------.
' |     Board Configuration                                                                             |
' `-----------------------------------------------------------------------------------------------------'
  _CLKMODE            = XTAL1 + PLL16x                  ' 80MHz System Clock Speed
  _XINFREQ            = 5_000_000

CON

  SIO_SPEED = 28800
  SBUFLEN = 128
  BREAKPOINT_SIZE = 10
  
OBJ

  sio: "Jazzed_SIO_Text"                                ' Serial IO
  jvm: "Jazzed_Jvm_Engine_009"                          ' JVM engine
  ops: "Jazzed_JVM_OpStrings_001"                       ' JVM debug opcode strings
  jem: "Jazzed_JVM_JemFile_003"                         ' The jem file for JVM to run
  jni: "Jazzed_JVM_jvmNative"                               ' native functions

VAR

  word gGetOverBreak                                    ' skip break once if set during isBreakPoint
  word gBreakpointHead                                  ' breakpoint array queue head
  word gBreakpointArray[BREAKPOINT_SIZE]                ' breakpoint array

PUB Main | last, ofp, omp, sbuf[SBUFLEN+1], printing, debug
{{              
 *******************************************************************************************************
 */

/**
 * Main entry point for standalone jvm testing.
 */
__ void main(); 
}}
  sio.start(31, 30, 0, SIO_SPEED)
  
  jvm.start
  jvm.setJvmStart(jem.ByteCodeStart)

  if getMaxJvmBreakpoints <> BREAKPOINT_SIZE
    sio.str(string($d,"Code err: Breakpoint size <> JVM."))
    repeat
    
  clearAllJvmBreakpoints
  printMenu
  
  jvm.jvmInit
  
  printing := 0

  repeat
    sio.Str(String($d,"JVM> "))
    result := sio.getch
    if result <> $D
      last := result
      ' if last is p and we're printing, get a new address.
      if last == "p" and printing
        printing := 0
    sio.out(last)
    sio.out(" ")
    case last
      "a": ' list disassembly of bytecodes from PC-2 instructions if possible to PC+20
        asmBytecode(-2, 20)
      "b": ' set breakpoint
        sio.Str(String(" JemPC $> "))
        result := getBreakAddr
        if result <> 0
          'sio.hex(result,4)
          setJvmBreakpoint(result+jvm.getJvmStart)
        result := 0
      "c": ' continue debugging
        jvm.skipJvmBreakPointOnce 
        repeat
          if sio.key
            quit
          if jvmDebug(1,debug) < 0
            quit
        jvmDumpBreak
      "C": ' clear all breakpoints
        clearAllJvmBreakpoints
      "d": ' delete a given breakpoint
        sio.Str(String("BP #> "))
        'result := getBreakAddr
        result := getBrkPtNum
        if result <> 0
          'sio.hex(result,4)
          'deleteJvmBreakpoint(result+jvm.getJvmStart)
          deleteJvmBrkPtByNum(result)
      "f": ' singlestep until frame changes
        ofp := jvm.getJvmFP
        repeat
          if jvmDebug(1,debug) < 0
            quit
          if sio.key <> 0
            sio.getch
            quit
        while (jvm.getJvmFP == ofp)
        jvmDumpBreak
      "h": ' dump heap
        jvm.ShowHeap
      "i": ' reinit jvm
        jvm.jvmInit
        jvm.resetJvmPc
      "j": ' jvm status
        jvm.dumpRegs
        jvm.showHeap
        jvm.showStack
      "l": ' list disassembly of bytecodes from PC-2 instructions if possible to PC+20
        asmBytecode(-2, 20)
      "L": ' list all breakpoints
        listJvmBreakpoints
      "m": ' singlestep until method changes
        omp := jvm.getJvmMP
        repeat
          if jvmDebug(1,debug) < 0
            quit
          if sio.key <> 0
            sio.getch
            quit
        while (jvm.getJvmMP == omp)
        jvmDumpBreak
      "n": ' singlestep over function
        omp := jvm.getJvmMP
        jvm.skipJvmBreakPointOnce
        repeat
          jvmDebug(1,debug)
          if sio.key <> 0
            sio.getch
            quit
        while (jvm.getJvmMP <> omp)
        'jvmDumpBreak
      "o": ' dump an object
        sio.Str(String(" ADDR $> "))
        sio.fgets(@sbuf, SBUFLEN)
        result := sio.strtohex(@sbuf)
        jvm.ShowObject(result)      
      "p": ' print memory
        if printing == 0
          sio.Str(String(" ADDR $> "))
          sio.fgets(@sbuf, SBUFLEN)
          result := sio.strtohex(@sbuf)
          dumpBytes(result,$80)
        else      
          dumpBytes(printing,$80)
        printing := result+$80
      "q": ' reboot
        reboot
      "r": ' run to a breakpoint
        jvm.jvmInit
        jvm.resetJvmPc
        sio.out($d)   
        repeat                                                                                                                                                                 
          if sio.key
            sio.getch
            quit
          if jvmDebug(1,debug) < 0
            quit
        jvmDumpBreak
      "s": ' single step into function
        jvm.skipJvmBreakPointOnce
        jvmDebug(1,debug)
        jvmDumpBreak
      "S": ' dump stack
        jvm.ShowStack
      "v": ' set callDebug verbosity
        jvm.setJvmCallDebug(1) 
      "V": ' set instruction level verbosity
        debug := 1 
        jvm.setJvmVerboseDebug(1)  
      "u": ' unset verbosity                                                                                                                        
        jvm.setJvmCallDebug(0)
        debug := 0 
        jvm.setJvmVerboseDebug(0)
      "x": ' x-amine memory location hex value
        sio.Str(String(" ADDR $> "))
        sio.fgets(@sbuf, SBUFLEN)
        result := sio.strtohex(@sbuf)
        sio.Str(string("ADDR "))
        sio.hex(result, 4)
        sio.Str(string(" = $"))
        sio.hex(word[result], 2)
      " ":
      other:
        printMenu

PUB jvmDebug(stp, verb) | pc, opcode

  pc := jvm.getJvmPC
  opcode := byte[pc]

  if jvm.isBreakpoint(pc)
    return -1
    
  result := jvm.jvmSpinDebug(stp, 0, ops.OpcodeNamePtr(opcode))
  
  handleNative(result)
      
  if verb
    jvmDumpBreak

  if jvm.getJvmMP == 0 or jvm.getJvmFP == 0
    return -1
    
  result := 0


PRI handleNative(code) | stk, ii, rcnt, val

  ' native methods are positive integers
  if code < 0
    return
{    
  sio.str(string($d,"Run Native "))
  sio.hex(code,2)
}
  rcnt := jvm.Pop
{
  sio.str(string(" P# "))
  sio.dec(rcnt)
  sio.out(" ")
}
  repeat while (rcnt-- > 0)
    val := jvm.GetVar(rcnt)
    'sio.hex(val,4)
    'sio.out(" ")
    result := jvm.GetHeapValuePtr(val,0)
    if result <> 0
      val := result
    jni.push(val)

  result := jni.do_native(code, @rcnt)
{
  sio.str(string(" R# "))
  sio.dec(rcnt)
}
  if rcnt > 0
    jvm.Push(result)
   
PUB jvmDumpBreak | pc, opcode
 pc := jvm.getJvmPC
  opcode := byte[pc]
  if jvm.isBreakpoint(pc)
    sio.str(string($d,"Breakpoint PC $"))
    sio.hex(pc,4)
    sio.str(string(" JemPC $"))
    sio.hex(pc-jvm.getJvmStart,4)
    sio.out($d)
  jvm.JvmSpinDumpBreak(opcode, ops.OpcodeNamePtr(opcode))

PUB asmBytecode(start, count) | ii, pc, mb, opc, bpt
  pc := jvm.getJvmPC
  mb := jvm.getJvmMP
  ii := mb-1
  sio.out($d)
  repeat while(++ii < pc+count)
    opc := byte[ii]
    bpt := jvm.isBreakpoint(ii)
    sio.str(string($d,"JPC "))
    if bpt
      sio.hex(bpt,1)
    else
      sio.out("-")
    sio.out(" ")
    sio.out("$")
    sio.hex(ii-jvm.getJvmStart,4)
    sio.out(" ")
    if(ii == pc)
      sio.out("*")
    else
      sio.out(" ")
    jvm.dumpOpcode(opc, ops.OpcodeNamePtr(opc))
    ii += ops.getOperandSize(opc) ' skip opcodes
  sio.out($d) 
    
PUB dumpBytes(addr,len) | ndx, bytes[17], bi, sz
{{
/**
 * dump bytes in 8 column table
 * @param addr is base address for dump
 * @param len is number of bytes to dump
 */
__ void dumpBytes(addr,len);
}}
  ndx := 0
  sz  := 16
  'str(string("Dump"))
  repeat while(ndx < len)
    if ndx <> 0 and (ndx // sz) <> 0
      sio.out(".")
      bytes[bi++ // sz] := byte[addr+ndx] 
    if ndx == 0 or (ndx // sz) == 0
      if ndx <> 0
        sio.out(" ")
        repeat bi from 0 to sz-1
          if (bytes[bi] => $20)
            sio.out(bytes[bi])
          else
            sio.out(".")
      sio.out($d)
      sio.hex(addr+ndx,4)
      sio.str(string(": "))
    sio.hex(byte[addr+ndx],2)
    ndx++
    
  if ndx <> 0
     sio.out(" ")
     repeat bi from 0 to sz-1
       if (bytes[bi] => $20)
         sio.out(bytes[bi])
       else
         sio.out(".")
       
PRI getBreakAddr | sbuf[SBUFLEN+1]
  sio.fgets(@sbuf, SBUFLEN)
  result := sio.strtohex(@sbuf)
  
  if result < jvm.getJvmMain
    sio.Str(String("PC < jem.out main address. Try again. "))
    sio.hex(result,4)
    sio.out(" ")
    sio.hex(jvm.getJvmMain,4)
    result := 0      
  elseif result > jvm.getJvmLength
    sio.Str(String("PC exceeds jem.out size. Try again. "))
    sio.hex(result,4)
    sio.out(" ")
    sio.hex(jvm.getJvmLength,4)
    result := 0      

PRI getBrkPtNum | sbuf[SBUFLEN+1]
  sio.fgets(@sbuf, SBUFLEN)
  result := sio.strtohex(@sbuf)
  if result < 1 or result > getMaxJvmBreakpoints
    sio.Str(String("Breakpoint range 1 to "))
    sio.dec(getMaxJvmBreakpoints)
    sio.Str(string(". Try again. "))
    sio.hex(result,4)
    sio.out(" ")
    sio.hex(jvm.getJvmMain,4)
    result := 0      
    
PRI printMenu
  sio.cls 
  sio.out($d)
  sio.out($d)
  sio.str(string($d,"JVM Human Interface Debug Help:"))
  'sio.str(string($d," a: show bytecode assembly"))
  sio.str(string($d," b: set breakpoint"))
  sio.str(string($d," c: continue debugging"))
  sio.str(string($d," C: clear all breakpoints"))
  sio.str(string($d," d: delete a given breakpoint"))
  sio.str(string($d," f: finish a method"))
  sio.str(string($d," h: dump heap"))
  sio.str(string($d," i: init JVM to reset state"))
  sio.str(string($d," j: show JVM status.")) 
  sio.str(string($d," l: list bytecode assembly"))
  sio.str(string($d," L: list all breakpoints"))
  sio.str(string($d," m: break on new method"))
  sio.str(string($d," n: next instruction in method"))
  sio.str(string($d," o: dump an object contents")) 
  sio.str(string($d," p: print memory range")) 
  sio.str(string($d," q: quit and reboot"))
  sio.str(string($d," r: run to a breakpoint"))  
  sio.str(string($d," s: single step instruction"))
  sio.str(string($d," S: dump stack"))
  sio.str(string($d," v: set callDebug verbosity")) 
  sio.str(string($d," V: set more detail verbosity")) 
  sio.str(string($d," u: unset verbosity")) 
  sio.str(string($d," x: examine one memory address")) 
  sio.str(string($d," Enter repeats last command."))
  'sio.str(string($d," Use 'n' to skip breakpoint."))
  sio.out($d)
  
PUB clearAllJvmBreakpoints | ii 
{{
/**
 * This is a debug control command that clears all breakpoints.
 * The breakpoint queue pointer will be 0 upon return.
 */
__ void clearAllJvmBreakpoints();
}}
  gBreakpointHead := 0
  repeat ii from 0 to BREAKPOINT_SIZE-1
    jvm.setJvmBrkPtByNumber(ii, 0)
    gBreakpointArray[ii] := 0


PUB deleteJvmBrkPtByNum(num) | ii
{{
/**
 * This is a debug control command that lets the user delete a breakpoint by number.
 * @param num is the breakpoint number
 * @returns zero if remove breakpoint failed
 */
__ int deleteJvmBrkPtByNum (int num);
}}
  deleteJvmBreakpoint(gBreakpointArray[num-1])
  result := 0
  
PUB deleteJvmBreakpoint(bppc) | ii, jj 
{{
/**
 * This is a debug control command that lets the user delete a breakpoint.
 * @param bppc is the program counter byte array instruction address.
 * @returns zero if remove breakpoint failed
 */
__ int deleteJvmBreakpoint(bppc);
}}
  result := 0

  ' roll through the queue
  ii := -1  
  repeat while(++ii < BREAKPOINT_SIZE)
    if gBreakpointArray[ii] == bppc
      ' shuffle breakpoints
      repeat while(++ii < BREAKPOINT_SIZE)
        'sio.str(string($d,"bp "))
        'sio.hex(ii, 4)
        gBreakpointArray[ii-1] := gBreakpointArray[ii]
      sio.str(string($d,"deleted bp "))
      sio.hex(ii // BREAKPOINT_SIZE + 1, 2)
      sio.str(string(" at "))
      sio.hex(bppc, 4)
      sio.str(string(" JemPC "))
      sio.hex(bppc-jvm.getJvmStart, 4)
      result := 1
      quit

  repeat ii from 0 to BREAKPOINT_SIZE-1
    jvm.setJvmBrkPtByNumber(ii, gBreakpointArray[ii])


PUB setJvmBreakpoint(bppc) 
{{
/**
 * This is a debug control command that lets the user set one breakpoint.
 * The breakpoint list is an array of BREAKPOINT_SIZE words that behaves like a queue.
 * Only BREAKPOINT_SIZE entries are allowed at one time. The newest entry will replace
 * the oldest entry.
 * @param pc is the program counter byte array instruction address.
 * @returns position of last breakpoint in queue  
 */
__ int setJvmBreakpoint(bppc);
}}
  sio.Str(String("PC $"))
  sio.hex(bppc,4)
  sio.out(" ")
  sio.Str(String($d,"JemPC $"))
  sio.hex(bppc-jvm.getJvmStart,4)
  sio.out(" ")
  result := gBreakpointArray[gBreakpointHead] := bppc
  jvm.setJvmBrkPtByNumber(gBreakpointHead, bppc)
  gBreakpointHead++
  result := gBreakpointHead //= BREAKPOINT_SIZE
  sio.Str(String("Breakpoint "))
  sio.dec(result)
                
PUB getJvmBreakpoint(num) 
{{
/**
 * This is a debug control command that lets the user get the pc of a breakpoint.
 * If the breakpoint input parameter is 0, get the last breakpoint, else get breakpoint specified.
 * The breakpoint number should be <= maximum allowed breakpoints.
 * If the number is greater than maximum, the breakpoint will be the mod of of the number.
 * @param num is the 1-base breakpoint number to get. If num is 0 get the last breakpoint set.
 * @returns pc of selected breakpoint or zero if breakpoint not found. 
 */
__ int getJvmBreakpoints(num);
}}
  if num > 0
    num--
    num //= BREAKPOINT_SIZE
    'result := gBreakpointArray[num]
    result := jvm.getJvmBrkPtByNumber(num)

PUB getMaxJvmBreakpoints 
{{
/**
 * This is a debug control command that tells user maximum number of breakpoints allowed.
 * @returns maximum number of breakpoints. 
 */
__ int getMaxJvmBreakpoints();
}}
  result := jvm.getMaxJvmBreakpoints

PUB skipJvmBreakPointOnce
{{
/**
 * Skip breakpoint once. required to get past breakpoint.
 */
__ void skipJvmBreakPointOnce(void);
}}
  gGetOverBreak := true
  
PUB listJvmBreakpoints | val, ii
{{
/**
 * list all breakpoints
 */
__ void listJvmBreakpoints();
}}
  result := 0
  repeat ii from 1 to BREAKPOINT_SIZE
    val := getJvmBreakpoint(ii)
    if val <> 0
      sio.str(string($d,"Breakpoint "))
      sio.dec(ii)
      sio.str(string(" @PC "))
      sio.hex(val, 4)
      sio.str(string(" JemPC "))
      sio.hex(val-jvm.getJvmStart, 4)
      result++

  if result == 0
    sio.str(string($d,"No Breakpoints Set"))
       
  