{{
  SIO_Text
  This module emulates features of TV_Text and VGA_Text using VT100 escape codes.
  For codes, see: http://pegasus.cs.csubak.edu/Tables_Charts/VT100_Escape_Codes.html
 }}
CON
  '_clkmode        = xtal1 + pll16x
  '_xinfreq        = 5_000_000

  LFCODE = $D
  CRCODE = $A
  MAX_X = 80 '32
  MAX_Y = 15
  MAX_CPOS = MAX_X*MAX_Y-1
  TABSIZE = 8
  VGADEBUG = 1

OBJ
  debug: "Jazzed_FullDuplexSerialPlus"
  vga  : "Jazzed_VGA_Text"
  
VAR
  byte buf[120]
  word curpos
  byte outcmd

PUB maintest | ii
' start up serio interface
  if VGADEBUG
    vga.start(16)
  start(31, 30, 0, 9600)
  CLS
  repeat
      
PUB start(opin, inpin, stp, baud) : rc
{{
/**
 * Start serial interface
 * @param opin is the serial output pin
 * @param inpin is the serial input pin
 * @param stp is the number of stop bits to use
 * @param baud is the baud rate to use
 * @returns =< 0 on failure
 */
}}
' start up serio interface
  if VGADEBUG
    vga.start(16)
    vga.out(0)
  curpos := 0
  outcmd := 0
  rc := debug.start(opin, inpin, stp, baud)
  CLS
  return rc

PUB stop
{{
/**
 * Stop serial interface
 */
}}
  debug.stop

PUB getCurPosition
  return curpos

PUB str(sp) | ii
{{
/**
 * send a string to serial interface
 * uses out instead of str to keep track of cursor
 * @param sp is string pointer
 */
}}
  ii := 0
  repeat while(byte[sp+ii])
    out(byte[sp+ii])
    ii++

PUB dec(value) | v
{{
/**
 * print a decimal value to serial interface
 * @param value is value to print
 */
}}
  if VGADEBUG
    vga.dec(value)    
  debug.dec(value)
  v := value
  repeat while v > 0
    v /= 10
    curpos++
  position(curpos)
  
PUB hex(value, digits)
{{
/**
 * print a hex value to serial interface
 * @param value is value to print
 * @param digits is number of digits to print
 */
}}
  if VGADEBUG
    vga.hex(value, digits)
  debug.hex(value, digits)
  curpos+=digits
  position(curpos)
    
PUB bin(value, digits)
{{
/**
 * print a binary value to serial interface
 * @param value is value to print
 * @param digits is number of digits to print
 */
}}
  if VGADEBUG
    vga.bin(value, digits)
  debug.bin(value, digits)
  curpos+=digits
  position(curpos)
    
PUB out(ch) | y, x, cp
{{
/**
 * out emulates TV/VGA out with VT100 codes
 *
 * @param c is char code to output based on table
 *
 * mon.out(val) screen printing
 * Output a character
 *
 *     $00 = clear screen
 *     $01 = home
 *     $08 = backspace
 *     $09 = tab (8 spaces per)
 *     $0A = set X position (X follows)
 *     $0B = set Y position (Y follows)
 *     $0C = set color (color follows)
 *     $0D = return
 *  others = printable characters
 */
}}
  if VGADEBUG
     vga.out(ch)
  if outcmd <> 0
    case outcmd
      $A: ' set X position
        x := ch//MAX_X
        y := curpos/MAX_X
        cp:= y*MAX_X+x
        position(cp)
      $B: ' set Y position
        x := curpos//MAX_X
        y := ch//MAX_Y
        cp:= y*MAX_X+x
        position(cp)
      $C:'coloring not possible on VT100 consume operand
      other: ' default should not get here
    outcmd := 0
  else 
    case ch
      $0: ' clear screen
        CLS
      $1: ' cursor home
        curHome
      $8: ' cursor backspace
        curBackSpace
      $9: ' cursor tab 8th character
        curTab
      $A: ' cursor set X position
         outcmd := ch
      $B: ' cursor set Y position
        outcmd := ch
      $C: ' cursor set color if possible
        outcmd := ch
      $D: ' cursor tty carriage return
        newline
      other:
           debug.tx(ch)
          curpos++

  
PUB setcolors(cptr)
{{
/**
 * coloring not possible on VT100
 * compatability stub
 * @param cptr for compatability
 */
}}
  'nada
  
PUB gets(s)
{{
/**
 * get string from serio ... potential for overrun here watch out
 * @note this method blocks until a newline is received
 * @param str is string received
 */
}}
  return debug.getstr(s)

PUB getch : ch
{{
/**
 * get char without enter from serio
 * @note this method blocks until a character is received
 * @returns character received
 */
}}
  ch := debug.rx

PUB key : ch
{{
/**
 * not implemented on multicogserialdebug
 * checks to see if a character is available at serial io
 * @note this method does not block
 * @returns character received
 */
}}
  ch := debug.rxkey

PUB rxcheck
{{
/**
 * checks for a char on input ... never blocks
 * @returns -1 if no char available else char $00 ..$ff
 */
}}
  return debug.rxcheck

PUB tx(ch)
{{
/**
 * checks for a char on input ... never blocks
 * @returns -1 if no char available else char $00 ..$ff
 */
}}
  if VGADEBUG
    if ch <> $A
      vga.out(ch)
  debug.tx(ch)

PUB getHex
{{
/**
 * get hex number from input ...
 * @returns hex value
 */
__ int getHex();
}}
  return debug.GetHex

PUB fgets(buff, len) | ii, ch
{{
/**
 * get string from input with echo limited by len
 * @param buff output buffer
 * @param len maximum length
 * @returns number of bytes read
 */
__ int gets(buff, len);
}}
  ii := 0
  repeat
    if ii > len-1
      byte[buff][ii-1] := 0
      return ii-1
    ch := debug.rx
    out(ch)
    if ch <> 8
      byte[buff][ii++] := ch
  while (ch <> $D)
  byte[buff][--ii] := 0
  result := ii

PUB StrToHex(stringptr)
  return debug.StrToHex(stringptr)
  
''
''========================== vt100 terminal commands ==========================
''
PRI newline
  position((curpos/MAX_X+1)*MAX_X)
  'curpos += MAX_X
  debug.str(string($A,$D))
       
PRI position(cp) | x,y
{{
/**
 * Move cursor to given position using vt100 code ^[[v;hH
 * @param x is the horizontal or column position
 * @param y is the vertical or row position
 */
}}
  return
  x := cp//MAX_X 
  y := cp/MAX_X
  curpos := cp
    
  debug.tx($1b)
  debug.str(string("["))
  debug.dec(y+1)
  debug.str(string(";"))
  debug.dec(x+1)
  debug.str(string("H"))

 if 0'VGADEBUG
    vga.out($A)
    vga.out(x)
    vga.out($B)
    vga.out(y)
    
PUB CLS
'' clear screen ^[[2J
  debug.tx($1b)
  debug.str(string("[2J"))
  if VGADEBUG
    vga.out(0)
  position(0)
    
PRI curHome
'' move cursor to top left "home" position ^[[H
  if VGADEBUG
    vga.out(1)
  position(0)
{
PRI curUp(num)
'' move cursor up num lines
  if curpos-MAX_X*num >= 0
    debug.tx($1b)
    debug.str(string("["))
    debug.dec(num)
    debug.str(string("A"))
    curpos -= MAX_X*num
    position(curpos)

PRI curDown(num)
'' move cursor down num lines
  if curpos+MAX_X*num < MAX_CPOS
    debug.tx($1b)
    debug.str(string("["))
    debug.dec(num)
    debug.str(string("B"))
    curpos += MAX_X*num
    position(curpos)

PRI curLeft(num)
'' move cursor left num columns
  if curpos > 0
    debug.tx($1b)
    debug.str(string("["))
    debug.dec(num)
    debug.str(string("D"))
    curpos--
    position(curpos)

PRI curRight(num)
'' move cursor right num columns
  if curpos < MAX_CPOS
    debug.tx($1b)
    debug.str(string("["))
    debug.dec(num)
    debug.str(string("C"))
    curpos++
    position(curpos)
}
PRI curBackSpace
'' move cursor left num columns
  if curpos > 0
    curpos--
    position(curpos)

PRI curTab | x,y,mx
  if curpos < MAX_X*MAX_Y
    x := curpos//MAX_X 
    y := curpos/MAX_X
    if (x < (MAX_X-TABSIZE)) and x//TABSIZE
      mx := x//TABSIZE
      curpos += (TABSIZE-mx)
      position(curpos)
 